package ij;
import java.io.*;
import java.util.*;
import java.applet.*;
import java.net.URL;
import java.awt.Color;
import ij.io.OpenDialog;
import ij.io.JpegEncoder;
import ij.util.Tools;
import ij.gui.Roi;
import ij.gui.Toolbar;
import ij.gui.ImageCanvas;
import ij.plugin.filter.ParticleAnalyzer;
import ij.plugin.filter.Analyzer;

/**
This class contains the ImageJ preferences, which are 
loaded from the "IJ_Props.txt" and "IJ_Prefs.txt" files.
@see ImageJ
*/
public class Prefs {

	public static final String PROPS_NAME = "IJ_Props.txt";
	public static final String PREFS_NAME = "IJ_Prefs.txt";
	public static final String DIR_IMAGE = "dir.image";
	public static final String FCOLOR = "fcolor";
	public static final String BCOLOR = "bcolor";
	public static final String ROICOLOR = "roicolor";
	public static final String JPEG = "jpeg";
	public static final String USE_POINTER = "pcursor";

	static String separator = System.getProperty("file.separator");
	static String prefsDir;
	static Properties prefs, props;
	static String imagesURL;
	static String homeDir; // directory containing IJ_Props.txt

	/** Finds and loads the ImageJ properties file.
		@return	an error message if "IJ_Props.txt" not found.
	*/
	public static String load(java.applet.Applet applet) {
		String jarDir = null;
		BufferedInputStream f = null;
		prefs = new Properties();
		props = new Properties(prefs);
				
		if (applet==null) { // running as application
			// look for IJ_Props.txt in user.dir
			String userDir = System.getProperty("user.dir");
			String userHome = System.getProperty("user.home");
			String osName = System.getProperty("os.name");
			if (osName.indexOf("Windows",0)>-1)
				prefsDir = userDir; //ImageJ folder on Windows
			else
				prefsDir = userHome; // Mac Preferences folder or Unix home dir 
			homeDir = userDir;
			boolean found = false;
			try {f = new BufferedInputStream(new FileInputStream(userDir+"/"+PROPS_NAME)); found = true;}
			catch (FileNotFoundException e) {}
			if (!found) { // look in user.home
				userHome = System.getProperty("user.home");
				homeDir = userHome;
				try {f = new BufferedInputStream(new FileInputStream(userHome+"/"+PROPS_NAME)); found = true;}
				catch (FileNotFoundException e) {}
			}
			if (!found) { // look in directory containing ij.jar
				String classPath = System.getProperty("java.class.path");
				int i1=0, i2=0;
				i2 = classPath.indexOf("ij.jar");
				if (i2>0) {
					i1 = classPath.lastIndexOf(System.getProperty("path.separator"), i2);
					if (i1<0)
						jarDir = classPath.substring(0, i2);
					else
						jarDir = classPath.substring(i1+1, i2);
					homeDir = jarDir;
					try {f = new BufferedInputStream(new FileInputStream(jarDir+"/"+PROPS_NAME)); found = true;}
					catch (FileNotFoundException e) {}
				}
			}
			if (!found) {
				String str = "";
				if (!userDir.equals(userHome))
					str = " or in\n  " + userHome;
				if (jarDir!=null && !jarDir.equals(userHome))
					str += " or in\n  " + jarDir;
				homeDir = null;
				return ("\"" + PROPS_NAME + "\" not found in\n  " + userDir + str);
			}
		}
		else {  // running as applet
			URL fileURL = null;
			try {
				fileURL = new URL(applet.getDocumentBase(), PROPS_NAME);
				//System.out.println(fileURL.toString());
				f = new BufferedInputStream(fileURL.openStream());
				URL url = new URL(applet.getDocumentBase(), "images/");
				imagesURL = url.toString();
				//System.out.println(imagesURL.toString());
			}
			catch (Exception e) {return ("Unable to open \"" + fileURL.toString() + "\"\n" + e);}
		}
		
		try {
			props.load(f);
			f.close();
		}
		catch (IOException e) {return("Error loading "+PROPS_NAME);}
		if (imagesURL==null)
			imagesURL = props.getProperty("images.location");
		if (applet==null)
			loadPreferences();
		return "";
	}
	
	/** Returns the URL for the ImageJ sample images. */
	public static String getImagesURL() {
		return imagesURL;
	}
	
	/** Returns the path to the directory containing IJ_Props.txt. */
	public static String getHomeDir() {
		return homeDir;
	}

	/** Finds an string in IJ_Props or IJ_Prefs.txt. */
	public static String getString(String key) {
		return props.getProperty(key);
	}
	
	/** Finds a boolean in IJ_Props or IJ_Prefs.txt. */
	public static boolean getBoolean(String key, boolean defaultValue) {
		if (props==null) return defaultValue;			
		String s = props.getProperty(key);
		if (s==null)
			return defaultValue;
		else
			return s.equals("true");
	}

	/** Finds an int in IJ_Props or IJ_Prefs.txt. */
	public static int getInt(String key, int defaultValue) {
		if (props==null) //workaround for Netscape JIT bug
			return defaultValue;			
		String s = props.getProperty(key);
		if (s!=null) {
			try {
				return Integer.decode(s).intValue();
			} catch (NumberFormatException e) {IJ.write(""+e);}
		}	
		return defaultValue;
	}

	/** Finds a color in IJ_Props or IJ_Prefs.txt. */
	public static Color getColor(String key, Color defaultColor) {
		int i = getInt(key, 0xaaa);
		if (i == 0xaaa)
			return defaultColor;
		return new Color((i >> 16) & 0xFF, (i >> 8) & 0xFF, i & 0xFF);
	}
	
	/** Opens the IJ_Prefs.txt file. */
	static void loadPreferences() {
		String path = prefsDir+separator+PREFS_NAME;
		try {
			InputStream is = new BufferedInputStream(new FileInputStream(path));
			prefs.load(is);
			is.close();
		} catch (Exception e) {
			return;
		}
	}

	/** Saves user preferences in the IJ_Prefs.txt properties file. */
	static void savePreferences() {
		try {
			Properties prefs = new Properties();
			String dir = OpenDialog.getDefaultDirectory();
			if (dir!=null)
				prefs.put(DIR_IMAGE, escapeBackSlashes(dir));
			prefs.put(ROICOLOR, Tools.c2hex(Roi.getColor()));
			prefs.put(FCOLOR, Tools.c2hex(Toolbar.getForegroundColor()));
			prefs.put(BCOLOR, Tools.c2hex(Toolbar.getBackgroundColor()));
			prefs.put(JPEG, Integer.toString(JpegEncoder.getQuality()));
			prefs.put(USE_POINTER, ImageCanvas.usePointer?"true":"false");
			ParticleAnalyzer.savePreferences(prefs);
			Analyzer.savePreferences(prefs);
			String path = prefsDir+separator+PREFS_NAME;
			savePrefs(prefs, path);
		} catch (Exception e) {
			CharArrayWriter caw = new CharArrayWriter();
			PrintWriter pw = new PrintWriter(caw);
			e.printStackTrace(pw);
			IJ.write(caw.toString());
			IJ.wait(10000);
		}
	}
			
	static void savePrefs(Properties prefs, String path) throws IOException{
		FileOutputStream fos = new FileOutputStream(path);
		BufferedOutputStream bos = new BufferedOutputStream(fos);
		PrintWriter pw = new PrintWriter(bos);
		pw.println("# ImageJ v"+ImageJ.VERSION+" Preferences");
		pw.println("# "+new Date());
		pw.println("");
		for (Enumeration e=prefs.keys(); e.hasMoreElements();) {
			String key = (String)e.nextElement();
			pw.print(key);
			pw.write('=');
			pw.println((String)prefs.get(key));
		}
		pw.close();
	}
	
	static String escapeBackSlashes (String s) {
		StringBuffer sb = new StringBuffer(s.length()+10);
		char[] chars = s.toCharArray();
		for (int i=0; i<chars.length; i++) {
			sb.append(chars[i]);
			if (chars[i]=='\\')
				sb.append('\\');
		}
		return sb.toString();
	}

}

