package ij.process;

/** 32-bit (float) image statistics, including histogram. */
public class FloatStatistics extends ImageStatistics {

	/** Constructs an ImageStatistics object from a FloatProcessor
		using the standard measurement options (area, mean,
		mode, min and max) and generating a 256 bin histogram. */
	public FloatStatistics(ImageProcessor ip) {
		this(ip, AREA+MEAN+MODE+MIN_MAX, 256);
	}

	/** Construct a FloatStatistics object from a FloatProcessor using
		the specified measurement options and number of histogram bins. */
	public FloatStatistics(ImageProcessor ip, int mOptions, int nBins) {
		this.nBins = nBins;
		this.width = ip.getWidth();
		this.height = ip.getHeight();
		setRoiLimits(ip);
		getStatistics(ip);
		if ((mOptions&MODE)!=0)
			getMode();
		if ((mOptions&CENTROID)!=0)
			getCentroid(ip);
	}

	void getStatistics(ImageProcessor ip) {
		double v;
		float[] pixels = (float[])ip.getPixels();
		histogram = new int[nBins];
		double sum = 0;
		double sum2 = 0;
		int[] mask = ip.getMask();
		
		// Find min and max
		double roiMin = Double.MAX_VALUE;
		double roiMax = Double.MIN_VALUE;
		if (mask!=null)
			// non-rectangular roi
			for (int y=roiY, my=0; y<(roiY+roiHeight); y++, my++) {
				int i = y * width + roiX;
				int mi = my * roiWidth;
				for (int x=roiX; x<(roiX+roiWidth); x++) {
					if (mask[mi++]==ip.BLACK) {
						v = pixels[i];
						if (v<roiMin) roiMin = v;
						if (v>roiMax) roiMax = v;
					}
					i++;
				}
			}
		else
			// rectangular roi or no roi
			for (int y=roiY; y<(roiY+roiHeight); y++) {
				int i = y * width + roiX;
				for (int x=roiX; x<(roiX+roiWidth); x++) {
					v = pixels[i++];
					if (v<roiMin) roiMin = v;
					if (v>roiMax) roiMax = v;
				}
			}
		min = roiMin; max = roiMax;
		histMin = min; histMax = max;

		// Generate histogram
		double scale = (nBins-1)/(histMax-histMin);
		binSize = (histMax - histMin)/nBins;
		if (mask!=null) { // non-rectangular roi
			pixelCount = 0;
			for (int y=roiY, my=0; y<(roiY+roiHeight); y++, my++) {
				int i = y * width + roiX;
				int mi = my * roiWidth;
				for (int x=roiX; x<(roiX+roiWidth); x++) {
					if (mask[mi++]==ip.BLACK) {
						v = pixels[i];
						pixelCount++;
						sum += v;
						sum2 += v*v;
						histogram[(int)(scale*(v-histMin))]++;
					}
					i++;
				}
			}
			min = roiMin; max = roiMax;
		}
		else {  // rectangular roi or no roi
			for (int y=roiY; y<(roiY+roiHeight); y++) {
				int i = y * width + roiX;
				for (int x=roiX; x<(roiX+roiWidth); x++) {
					v = pixels[i++];
					sum += v;
					sum2 += v*v;
					histogram[(int)(scale*(v-histMin))]++;
				}
			}
			pixelCount = roiWidth*roiHeight;
		}
		
		mean = sum/pixelCount;
		calculateStdDev(pixelCount, sum, sum2);
	}

	void getMode() {
        int count;
        maxCount = 0;
        for (int i = 0; i < nBins; i++) {
        	count = histogram[i];
            if (count > maxCount) {
                maxCount = count;
                mode = i;
            }
        }
	}

}