package ij.process;

/** 16-bit image statistics, including histogram. */
public class ShortStatistics extends ImageStatistics {

	protected boolean unsigned;

	/** Construct an ImageStatistics object from a ShortProcessor
		using the standard measurement options (area, mean,
		mode, min and max) and generating a 256 bin histogram. */
	public ShortStatistics(ImageProcessor ip) {
		this(ip, AREA+MEAN+MODE+MIN_MAX, 256);
	}

	/** Constructs a ShortStatistics object from a ShortProcessor using
		the specified measurement options and number of histogram bins.
		The number of bins is currently fixed at 256. */
	public ShortStatistics(ImageProcessor ip, int mOptions, int bins) {
		this.width = ip.getWidth();
		this.height = ip.getHeight();
		unsigned = ip.isUnsigned();
		setRoiLimits(ip);
		nBins = 256;
		getStatistics(ip);
		if ((mOptions&MODE)!=0)
			getMode();
		if ((mOptions&CENTROID)!=0)
			getCentroid(ip);
		if ((mOptions&CENTER_OF_MASS)!=0)
			getCenterOfMass(ip);
	}

	public void getStatistics(ImageProcessor ip) {
		int v;
		short[] pixels = (short[])ip.getPixels();
		histogram = new int[nBins];
		double sum = 0;
		double sum2 = 0;
		int[] mask = ip.getMask();
		
		// Find min and max
		int roiMin = Integer.MAX_VALUE;
		int roiMax = Integer.MIN_VALUE;
		if (mask!=null)
			// non-rectangular roi
			for (int y=roiY, my=0; y<(roiY+roiHeight); y++, my++) {
				int i = y * width + roiX;
				int mi = my * roiWidth;
				for (int x=roiX; x<(roiX+roiWidth); x++) {
					if (mask[mi++]==ip.BLACK) {
						v = pixels[i];
						if (v<roiMin) roiMin = v;
						if (v>roiMax) roiMax = v;
					}
					i++;
				}
			}
		else
			// rectangular roi or no roi
			for (int y=roiY; y<(roiY+roiHeight); y++) {
				int i = y * width + roiX;
				for (int x=roiX; x<(roiX+roiWidth); x++) {
					v = pixels[i++];
					if (v<roiMin) roiMin = v;
					if (v>roiMax) roiMax = v;
				}
			}
		min = roiMin; max = roiMax;
		histMin = roiMin; histMax = roiMax;

		// Generate histogram
		double scale = (float)((nBins-1)/(histMax-histMin));
		int hMin = (int)histMin;
		binSize = (histMax - histMin)/nBins;
		if (mask!=null) { // non-rectangular roi
			pixelCount = 0;
			for (int y=roiY, my=0; y<(roiY+roiHeight); y++, my++) {
				int i = y * width + roiX;
				int mi = my * roiWidth;
				for (int x=roiX; x<(roiX+roiWidth); x++) {
					if (mask[mi++]==ip.BLACK) {
						v = pixels[i];
						pixelCount++;
						sum += v;
						sum2 += v*v;
						histogram[(int)(scale*(v-hMin))]++;
					}
					i++;
				}
			}
			// min = roiMin; max = roiMax;
		}
		else {  // rectangular roi or no roi
			for (int y=roiY; y<(roiY+roiHeight); y++) {
				int i = y * width + roiX;
				for (int x=roiX; x<(roiX+roiWidth); x++) {
					v = pixels[i++];
					sum += v;
					sum2 += v*v;
					histogram[(int)(scale*(v-hMin))]++;
				}
			}
			pixelCount = roiWidth*roiHeight;
		}
		
		mean = sum/pixelCount;
		if (unsigned)
			{min+=32768; max+=32768; histMin+=32768; histMax+=32768; mean+=32768;}
		calculateStdDev(pixelCount, sum, sum2);
	}


	void getMode() {
        int count;
        maxCount = 0;
        for (int i = 0; i < nBins; i++) {
        	count = histogram[i];
            if (count > maxCount) {
                maxCount = count;
                mode = i;
            }
        }
	}

	void getCenterOfMass(ImageProcessor ip) {
		short[] pixels = (short[])ip.getPixels();
		int[] mask = ip.getMask();
		boolean unsigned = ip.isUnsigned();
		int count=0, v, i, mi;
		double xsum=0.0, ysum=0.0;
		for (int y=roiY,my=0; y<(roiY+roiHeight); y++,my++) {
			i = y*width + roiX;
			mi = my*roiWidth;
			for (int x=roiX; x<(roiX+roiWidth); x++) {
				if (mask==null || mask[mi++]==ip.BLACK) {
					v = pixels[i];
					if (unsigned) v+=32768;
					count += v;
					xsum += x*v;
					ysum += y*v;
				}
				i++;
			}
		}
		xCenterOfMass = xsum/count+0.5;
		yCenterOfMass = ysum/count+0.5;
	}

}